/*******************************************************************************

    TACOMETRO_2 C/ DISPLAYS DE 7 SEG (4 DGITOS)


    AUTOR: CLAUDIO LRIOS

    DATA: 25/06/2020

    COM PIC 16F628A

    FAIXA DE TRABALHO: DE 10 A 9999 RPM

    DISPLAYS DE 7 SEGUIMENTOS (OPCIONAL CATODO OU ANODO COMUM)
    IMPORTANTE: USE O ESQUEMA CORRETO CONFORME A OPO)


    OBS. MONTAGEM  DE NATUREZA DIDTICA

    ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG LARIOS.TECNOLOGIA.WS


*******************************************************************************/

//   #define ANODO_COMUM  //descomente para anodo comum

// Comente o acima para catodo comum
//=============================================================================/


#include <16F628A.h>
#use delay(clock=4000000)//USE CRISTAL EXTERNO DE 4 MHZ
#fuses NOWDT,HS, NOPUT, NOPROTECT, BROWNOUT, MCLR, NOLVP, NOCPD


//BYTE
#byte portb = 0x06
#byte porta = 0x05
#byte trisa = 0x85
#byte trisb = 0x86
#byte t1con = 0x10
#byte tmr1h = 0x0f
#byte tmr1l = 0x0e
#byte tmr0  = 0x01
#byte option_reg = 0x81
#byte cmcom = 0x1f
#byte intcon = 0x0b
//BIT
#bit  t1on  = 0x10.0
#bit  tmr1if  = 0x0c.0
#bit  TMR1IF  = 0x0c.0
#bit tmr1ie   = 0x8c.0 //on/off interrupo de overflow do tmr1
#bit t0if     = 0x0b.2
#bit  t0ie    = 0x0b.5
#bit  sensor = 0x5.4


//constantes para converso 7 seguimentos

//                         PGFEDCBA   SEGUIMENTOS
//                   PORTB 76543210
int8 const NUM[] = {     0B00111111,//0
                         0B00000110,//1
                         0B01011011,//2
                         0B01001111,//3
                         0B01100110,//4
                         0B01101101,//5
                         0B01111101,//6
                         0B00000111,//7
                         0B01111111,//8
                         0B01101111 //9
                                        };

 //variveis globais
 int8 dig3=0,dig2=0,dig1=0,dig0=0;
 int8 dig3a=0,dig2a=0,dig1a=0,dig0a=0;
 int16 valor=0,waux,valor_ant=0;
 int1 fok=0;
 int8 ger=0, cnt=0;
 int32 soma32=0;
 /******************************************************************************
           MULTIPLEXA OS DISPLAYS
******************************************************************************/

 void multiplexa(){
    static int8 vez;

    if(fok){
     dig0=dig0a;
     dig1=dig1a;
     dig2=dig2a;
     dig3=dig3a;
     fok=0;     
     ger=0;//nova leitura da rotao    
    }
   

   if(++vez>3) vez=0;
    porta&=0xf0;//apaga display (resetando)
    #ifdef ANODO_COMUM
    portb=0xff;
    #else
    portb=0;
    #endif

    switch (vez){

     case 0: {
       portb=num[dig3];
       #ifdef ANODO_COMUM
       portb=~portb;
       #endif
       if(dig3) porta= porta|8;//apaga zeros a esquerda
       break;}
     case 1: {
       portb=num[dig2];
       #ifdef ANODO_COMUM
       portb=~portb;
       #endif
       if(dig3 || dig2) porta= porta|4;//apaga zeros a esquerda
       break;}
     case 2: {
       portb=num[dig1];
       #ifdef ANODO_COMUM
       portb=~portb;
       #endif
       if(dig3 || dig2 || dig1) porta= porta|2;//apaga zeros a esquerda
       break;}
     case 3: {
       portb=num[dig0];
       #ifdef ANODO_COMUM
       portb=~portb;
       #endif
       porta= porta|1; break;}
    }
 }

//==============================================================================
//         ROTINA DE INTERRUPO DO TIMER 1
//==============================================================================

#INT_TIMER0
void isr_tmr0(){


   if(ger==1){
     t1on=0; //desliga timer 1
     ger=2;
   }

   if(ger==0){
    tmr1l=0;//zera tmr1 low
    tmr1h=0;//zera tmr1 high
    tmr1if=0;//apaga flag overflow do timer1
    t1con=1;//liga timer1    
    ger=1;
   }

   tmr0=255;//carga do timer 0
}
//==============================================================================
//         ROTINA MAIN
//==============================================================================
void main() {

   option_reg=0b11101000; //timer 0 contando sinal externo sem prescaller
   cmcom=7;//desliga comparadores analgicos internos
   trisa=0b11110000;//sentido fixo das portas
   trisb=0;//idem
   porta=0;//desliga displays anodo ou catodo
   t1con=0;
   ger=0;
   tmr1l=0;
   tmr1h=0;
   tmr1if=0;
   t0if=0;
   tmr0=255;
   intcon=0b11100000; //liga interrupes gerais e perifricos e tmr0
//==============================================================================
//         LAO DE REPETIO PRINCIPAL
//==============================================================================
   for(;;){

   
       if((TMR1IF==1) && (ger==1)){//toda vez que for 1 soma 0xffff
          soma32 = soma32 + 0xffff; // estouro no timer 1
          TMR1IF = 0;//reseta flag de overflow do timer1
       }

       if (ger==2){

        valor = TMR1H;//obtm resultado low do timer1
        valor = valor<<8;//desloca 8 bits para esquerda
        valor = valor | TMR1L;//obtm resultado high do timer1
        soma32 = soma32 + valor;// obtm valor total da contagem do intervalo

        if (soma32 <= 6000000){
          soma32 = 1000000000 / soma32;// obtm o inverso do perodo=frequncia
          soma32 = soma32 * 60;// multiplica por 60 para obter em rpm

          valor = soma32 / 1000;//divide por mil para ajustar resultado
          
          soma32=0;//prepara para prxima leitura

          if(valor > 9999) valor = 0;//limita rpm mxima a 9999 rpm
          if(valor<30) valor = 0; //limita rpm mnima a 30 rpm

          if(valor>100){// se leitura maior que 100 rpm
            for(waux=0; waux<valor; waux++){// intercala tempo entre leituras proporcional a rpm
              multiplexa();//realiza multiplexao para amenizar efeito cintilatrio nos displays
              delay_us(10);//tempo adicional de ajuste
            }
          }
          
          dig3a = valor / 1000; //converte para decimal nos dgitos auxiliares
          valor = valor % 1000;
          dig2a = valor / 100;
          valor = valor % 100;
          dig1a = valor / 10;
          dig0a = valor % 10 ;
          fok = 1; //sinaliza que houve um valor novo para mostrar nos dgitos

        }
      }//if(ger==2)

      multiplexa();
      delay_ms(1);

    }//for(;;)
}//main()
//==============================================================================
//                            Fim do  Programa
//==============================================================================
